%FRF: Plot Frequency Response Function
%
% todo: change options into struct
%       make suitable for models with freq in hz
%
%   Usage:   frf(sys1,'color/linetype/linewidth/marker/markersize',sys2,'color/linetype/linewidth/marker/markersize',...
%            ...{fmin,fmax,magmin,magmax,phasemin,phasemax,wraponoff,type})
%
%   Inputs:  You can use custom colors, e.g. lightgrey, pink, etc. (see line ~315)
%            To use the . marker give \. as marker (to distinguist from dots in linewidth specs) 
%            example strings: 'lightgrey--1.5o12'    lightgrey dashed line with circle marker of size 12 
%                             'orange\.'             orange solid line, dot marker 
%                             'b*10'                 blue markers size 10, no line
%            All elements in the options-struct are optional
%            fmin/fmax in Hz
%            magmin/magmax in dB
%            phasemin/phasemax in degrees
%            wraponoff: [ 'on' | 'off' (default) ]
%            type: [ 'mag' | 'phase' | 'magphase' (default) ]
%
%   Outputs: Plots the frf in the current figure.
%            Note that the xticklabels might overlap the xlabel in the
%            figure. In an exported pdf file they will be correct.
%
%   See also FRFMAG, FRFPHASE, FRFHOLD, SETPLOT, SAVER

%   Changes:
%   20110803 - Initial version (Rob Hoogendijk)
%	20110914 - Changed rotation and alignment of Out: i labels (Michael Ronde)
%   20110916 - Small bugfix (Out: i labels for every system) (Michael Ronde)
% 	20111013 - Added axes handles as optional output (Michael Ronde)
%   20111016 - Bugfix for panning and zooming of MIMO plots (Michael Ronde)
%   20120203 - Cleanup and add linewidth/markersize option (Rob Hoogendijk/Michael Ronde)
%   20120405 - Added 'hold' use: 'frfhold on' and 'frfhold off' between calls of frf (Rob Hoogendijk)

%
%   Rob Hoogendijk (2011-08-3)
%   ________________________________
%   Eindhoven University of Technology
%   Dept. of Mechanical Engineering
%   Control Systems Technology group
%   PO Box 513, WH -1.126
%   5600 MB Eindhoven, NL
%   T +31 (0)40 247 4227
%   F +31 (0)40 246 1418
%   E r.hoogendijk@tue.nl

function varargout = frf(varargin)
%settings (available if setplot.m is installed)
fontname='times';           %of the labels, axes, legends
fontsize=10;                %of the labels, axes, legends
rowspacing=[];              %in centimeter, [] gives minimal spacing
colspacing=[];              %in centimeter, [] gives minimal spacing
defaultwrapsetting=[];      %wrap the phase to a range -180,180 or not (value gives top phase)
                            %empty means dont wrap
radunits=0;                 %0: use Hz 1: use radians/sec
npoints=5000;               %number of frequency points

%% read inputs
nsys=0; %couter for number of systems
optionsfound=0;
defaultcolors='bgrcmkybgrcmkybgrcmkybgrcmkybgrcmkybgrcmky';
for ii=1:length(varargin)
    inputii=varargin{ii};
    %detect systems, set default styles
    if issystem(inputii)
        nsys=nsys+1;
        systems{nsys}.lti=inputii;
        systems{nsys}.noutputs=size(systems{nsys}.lti,1);
        systems{nsys}.ninputs=size(systems{nsys}.lti,2);
        systems{nsys}.linetype='-'; %default style is only a line
        systems{nsys}.color=defaultcolors(nsys); %default colors
        systems{nsys}.linewidth=1.5; %default linewidth
        systems{nsys}.markersize=6;
        systems{nsys}.marker='none';
    end
    %detect formatting
    if ischar(inputii) 
        systems{nsys}.style=inputii;
        temp = cst_plotstr_to_struct(inputii);       
        if(~isempty(temp.color)),      systems{nsys}.color      = temp.color;      end
        if(~isempty(temp.linetype)),   systems{nsys}.linetype   = temp.linetype;   end
        if(~isempty(temp.linewidth)),  systems{nsys}.linewidth  = temp.linewidth;  end
        if(~isempty(temp.marker)),     systems{nsys}.marker     = temp.marker;     end
        if(~isempty(temp.markersize)), systems{nsys}.markersize = temp.markersize; end
        
    end
    %detect options
    if iscell(inputii)
        options=inputii;
    end
end
%try to process options (if not given set default values)
try fmin     = options{1};   catch, fmin=[];             end;
try fmax     = options{2};   catch, fmax=[];             end;
try magmin   = options{3};   catch, magmin=[];           end;
try magmax   = options{4};   catch, magmax=[];           end;
try phasemin = options{5};   catch, phasemin=[];         end;
try phasemax = options{6};   catch, phasemax=[];         end;
try wrapping = options{7};   catch, wrapping=defaultwrapsetting; end;
try plottype = options{8};   catch, plottype='magphase'; end;

if(~radunits)
    fmin=fmin*2*pi;
    fmax=fmax*2*pi;
end

%% create_axes_handles
noutputs=0;ninputs=0;%init
for ii=1:nsys %detect max system size
    maxnoutputs=max( noutputs , systems{ii}.noutputs );
    maxninputs=max( ninputs , systems{ii}.ninputs );
end
% determine plottype
switch plottype 
    case 'magphase', plotmag=1;plotphase=1;
    case 'mag',      plotmag=1;plotphase=0;
    case 'phase',    plotmag=0;plotphase=1; 
end;
%generate plot handles
udata=get(gcf,'UserData');

try %try to read handles from userdata of figure to enable subsequent plotting    
    handles=udata.handles;
    phasehandles=udata.phasehandles;
    maghandles=udata.maghandles;   
    if(~strcmp(udata.nextplot,'add'))
        clf;error;
    end
    %todo: check size/type of plot and use this to check if its feasible to
    %plot in the same graph else error or maybe add subplots
catch    
    handles=zeros((plotmag+plotphase)*maxnoutputs,maxninputs);
    maghandles=zeros(maxnoutputs,maxninputs);
    phasehandles=zeros(maxnoutputs,maxninputs);
    for nn=1:maxnoutputs
        for mm=1:maxninputs
            if(plotmag && plotphase)
                handles(2*nn-1,mm)=subplot(2*maxnoutputs,maxninputs,(2*(nn-1))*maxninputs+mm);hold on;grid on;       
                maghandles(nn,mm)=handles(2*nn-1,mm);%mag
                handles(2*nn,mm)=subplot(2*maxnoutputs,maxninputs,(2*(nn-1)+1)*maxninputs+mm);hold on;grid on;
                phasehandles(nn,mm)=handles(2*nn,mm);%phase
            else
                handles(nn,mm)=subplot(maxnoutputs,maxninputs,(nn-1)*maxninputs+mm);hold on;grid on;
                if(plotmag)
                    maghandles(nn,mm)=handles(nn,mm);%mag
                    phasehandles=[];
                else
                    phasehandles(nn,mm)=handles(nn,mm);%phase
                    maghandles=[];
                end
            end            
        end        
    end
    udata.handles=handles;
    udata.maghandles=maghandles;
    udata.phasehandles=phasehandles;
    set(gcf,'UserData',udata);
end



%% plot_frf
%plotting
for ii=1:nsys
    sys=systems{ii};
    
    %determine frequency range to plot
    %get freq vector from frd or from bode()
    if(isa(sys.lti,'frd') || isa(sys.lti,'idfrd'))
        if( strcmp(get(sys.lti,'FrequencyUnit'),'Hz') ) 
            wtemp=sys.lti.frequency*2*pi;
        else
            wtemp=sys.lti.frequency;
        end
    else
        if(~isempty(fmin) && ~isempty(fmax)  )
            [~,~,wtemp]=bode(sys.lti,{fmin,fmax});
        else
            [~,~,wtemp]=bode(sys.lti);
        end        
    end
       
    if(~isempty(fmin))%if fmin is given seach for first point in sys.w that is larger than fmin
        istart=find(wtemp>fmin,1);                        
    else%fmin is not given: take first point in the frd freq vector
        istart=1;
    end
    if(~isempty(fmax))%if fmax is given seach for last point in sys.w that is smaller than fmax
        iend=find(wtemp<fmax,1,'last');                       
    else%fmax is not given, take last point in the frd freq vector
        iend=length(wtemp);
    end
    sys.w=wtemp(istart:iend);
    [sys.mag,sys.phase,sys.w]=bode(sys.lti,sys.w);    
                
    if(~isempty(wrapping));
        sys.phase=wrap(sys.phase,wrapping);
    end
    if(radunits)
        freq=sys.w;
    else
        freq=sys.w/2/pi;
    end
    
    for nn=1:sys.noutputs
        for mm=1:sys.ninputs            
            %magnitude plots
            if(plotmag)
                semilogx(handles((plotmag+plotphase)*nn-plotphase,mm),freq,db(squeeze(sys.mag(nn,mm,:))),'linestyle',sys.linetype,'color',sys.color,'linewidth',sys.linewidth,'marker',sys.marker,'markersize',sys.markersize);                                           
            end
            %phase plots
            if(plotphase)                
                semilogx(handles((plotmag+plotphase)*nn,mm),freq,squeeze(sys.phase(nn,mm,:)),'linestyle',sys.linetype,'color',sys.color,'linewidth',sys.linewidth,'marker',sys.marker,'markersize',sys.markersize);                                 
            end
        end
    end
end
set(handles(:),'xscale','log','box','on');


%axes limits settings
ha=handles(:);
for ii=1:length(ha)
    hc=get(ha(ii),'children');               
    for jj=1:length(hc)
        maxx(ii,jj)=max(get(hc(jj),'xdata'));
        minx(ii,jj)=min(get(hc(jj),'xdata'));
    end
end   
if(isempty(fmin)), fmin=min(min(minx)); 
else if(~radunits), fmin=fmin/2/pi;end
end
if(isempty(fmax)), fmax=max(max(maxx));
else if(~radunits), fmax=fmax/2/pi;end
end
if(fmin~=fmax)
    set(ha,'xlim',[fmin fmax]);
end
%magnitude axis
ha=maghandles(:);
if(~isempty(ha))
    for ii=1:length(ha)
        hc=get(ha(ii),'children');               
        for jj=1:length(hc)
            ydata=get(hc(jj),'ydata');
            index_not_inf=find(ydata<inf &ydata>-inf);
            if(~isempty(index_not_inf))%cope with infinite boundaries...
                maxy(ii,jj)=max(ydata(index_not_inf));
                miny(ii,jj)=min(ydata(index_not_inf));
            else
                maxy(ii,jj)=-300;
                miny(ii,jj)=300;
            end 
        end
    end
    ws1=0;ws2=0;%variables to check whether magmin,magmax is given or computed
    if(isempty(magmin)), magmin=min(min(miny)); ws1=1; end
    if(isempty(magmax)), magmax=max(max(maxy)); ws2=1; end
    if(magmin~=magmax )
        ws=(magmax-magmin)*0.05; %fancy whitespace        
        set(ha,'ylim',[magmin-ws*ws1 magmax+ws*ws2]);
    end
end
%phase axis
ha=phasehandles(:);
for ii=1:length(ha)
    hc=get(ha(ii),'children');               
    for jj=1:length(hc)
        maxy(ii,jj)=max(get(hc(jj),'ydata'));
        miny(ii,jj)=min(get(hc(jj),'ydata'));
    end
end
if(isempty(phasemin)), phasemin=min(min(miny)); end
if(isempty(phasemax)), phasemax=max(max(maxy)); end
if(phasemin~=phasemax)
    ws=(phasemax-phasemin)*0.1; %fancy whitespace
    set(ha,'ylim',[phasemin-ws phasemax+ws]);
end




%% add labels, ticks etc.
%detect mimo or siso
if(maxnoutputs>1||maxninputs>1)
    mimo=1;
else
    mimo=0;
end
%set ticks
%remove x ticks
set(handles(1:end-1,:),'XTickLabel',{});
set(handles(:,:),'XTick',[1 10 100 1000]);
%remove y ticks 
set(handles(:,2:end),'YTickLabel',{});
%set ticks for phase plots
nticks=4; 
dtick=90;%initial dtick
while floor((phasemax-phasemin)/dtick)<(nticks-1) && dtick>2
    dtick=0.5*dtick;
end
while floor((phasemax-phasemin)/dtick)>(nticks-1)
    dtick=2*dtick;
end
set(phasehandles,'ytick',-36000:dtick:36000);  

%set labels
magtag='Magnitude [dB]';
phasetag='Phase[deg.]';
if(radunits),freqtag='Frequency [rad/s]';else,freqtag='Frequency [Hz]';end;     
if(mimo)
    % one x and y label for mimo systems
    if(isempty(findall(gcf,'type','axes','visible','off')))%if invisible axis does not exist
        ax=axes('position',[0.1,0.1,1,1],'visible','off');
        xlab=get(ax,'xlabel');       ylab=get(ax,'ylabel');
        if(plotmag && plotphase),set(ylab,'string',[magtag ', ' phasetag],'visible','on');end
        if(plotmag && ~plotphase),set(ylab,'string',magtag,'visible','on');end
        if(~plotmag && plotphase),set(ylab,'string',phasetag,'visible','on');end
        set(xlab,'string',freqtag,'visible','on');
        % Change order of the axes, otherwise axes for global labels is in front,
        % i.e. other axes cannot be panned/zoomed etc
        set(gcf,'Children',flipud(get(gcf,'Children')))
    end
else   
   %set x handles to freq 
   xhandles=get(handles(end,:),'XLabel');  
   for ii=1:length(xhandles)
       if(length(xhandles)==1)%then its not a cell array
           set(xhandles,'string',freqtag);
       else%it is a cell array
           set(xhandles{ii},'string',freqtag);
       end              
   end
   %set y handles to mag or phase
   set(get(maghandles,'ylabel'),'string',magtag);
   set(get(phasehandles,'ylabel'),'string',phasetag); 
end

%in/out labels
if(mimo)
    row1hnd=handles(1,:);
    for ii=1:length(row1hnd)
        title(row1hnd(ii),['In:' num2str(ii)],'fontsize',8);
    end
    colendhnd=handles(:,end);
    for ii=1:length(colendhnd)        
        outputnumber=ceil(ii/(plotmag+plotphase));
        ylabel(colendhnd(ii),['Out:' num2str(outputnumber)],'fontsize',8,'tag','right');
    end
end



%% change_figure_settings
%set(gcf,'color',[0.57 0.76 0.95]); %fancy blue

%axis positions 
if exist('setplot','file')==2
    setplot([], [], {0.5,1.0,rowspacing,1.1,0.25,colspacing}, fontsize, fontname);
else
    disp('Install setplot.m for optimal viewing of your frf.')
end

%linking axis
for ii=1:size(handles,2)%link x axes
    eval(['global link_fig' num2str(gcf) '_xaxis_' num2str(ii)]);
    eval(['link_fig' num2str(gcf) '_xaxis_' num2str(ii) '=linkprop(handles(:,ii),''XLim'' );']);
end
for ii=1:size(handles,1)%link y axes
    eval(['global link_fig' num2str(gcf) '_yaxis_' num2str(ii)]);
    eval(['link_fig' num2str(gcf) '_yaxis_' num2str(ii) '=linkprop(handles(ii,:),''YLim'' );']);
end

%% set outputs if requested
if nargout>0
  varargout{1}=handles;
end

%% check if all axes have same limits, else give warning (debug)
ha=maghandles(:);
for(ii=1:length(ha))
    yl(ii,:)=get(ha(ii),'ylim');
    if(ii>1)
        if(yl(ii,1)~=yl(ii-1,1) || yl(ii,2)~=yl(ii-1,2) || yl(ii,1)==-inf || yl(ii,2)==inf )
            disp('warning, unqual or inf axis limits reported by frf.m');
        end
    end
end
ha=phasehandles(:);
for(ii=1:length(ha))
    yl(ii,:)=get(ha(ii),'ylim');
    if(ii>1)
        if(yl(ii,1)~=yl(ii-1,1) || yl(ii,2)~=yl(ii-1,2) || yl(ii,1)==-inf || yl(ii,2)==inf )
            disp('warning, unqual or inf axis limits reported by frf.m');
        end
    end
end

    
end%main function

%% private functions
function pw=wrap(p,top)
    if nargin == 1,
        p = p-(round(p/360)*360);
    elseif nargin == 2,
        p	= p+180-top;
        pw	= p-(round(p/360)*360)-180+top;
    end
end

function plotobj = cst_plotstr_to_struct(plotstr) 
%          b     blue          \.    point             -     solid 
%          g     green         o     circle             :     dotted 
%          r     red           x     x-mark             -.    dashdot 
%          c     cyan          +     plus               --    dashed 
%          m     magenta       *     star             (none)  no line 
%          y     yellow        s     square 
%          k     black         d     diamond 
%          w     white         v     triangle (down) 
%                              ^     triangle (up) 
%                              <     triangle (left) 
%                              >     triangle (right) 
%                              p     pentagram 
%                              h     hexagram 
plotobj.color = '';
plotobj.linetype = 'none';
plotobj.linewidth = '';
plotobj.marker = '';
plotobj.markersize = '';

% Custom colors definitions
    custom= {...
        'lightgrey', [0.80 0.80 0.80];
        'midgrey',   [0.60 0.60 0.60];
        'darkgrey',  [0.40 0.40 0.40];
        'orange',    [1.00 0.50 0.00];
        'pink',      [1.00 0.50 0.60];
        'purple',    [0.80 0.00 1.00];
        'darkgreen', [0.00 0.70 0.00]; };

    %% Color options
    % Check if custom color option is found
    for j=1:size(custom,1)
        k=regexp(plotstr,custom{j,1}, 'once');
        if ~isempty(k)
            plotobj.color = custom{j,2};   
            plotstr = strrep(plotstr,custom{j,1},'');
        end
    end
    %else look for matlab color
    if(isempty(plotobj.color))
         plotobj.color = regexp(plotstr,'[rgbcymkw]','match','once');
    end    
    
    %%linestyle
    plotobj.linetype=regexp(plotstr,'(--)|(-\.)|(:)|(-)','match','once');     
    
    %%marker
    [plotobj.marker markerindex]=regexp(plotstr,'[ox+*sd<>v^ph]','match','once');        
    % for dot marker use \.   
    temp=regexp(plotstr,'\\.');
    if(~isempty(temp))
        plotobj.marker='.';
        markerindex=temp;
    end
    
    %set linetype to none if a marker is specified, but no linestyle
    if( ~isempty(plotobj.marker) && isempty(plotobj.linetype) )
        plotobj.linetype='none';
    end
    
    %%numeric entries
    [nums, index] = regexp(plotstr,'\d+\.*\d*','match');
    for ii=1:length(nums)
       if(index(ii)>markerindex)
           plotobj.markersize=str2double(nums{ii});
       else
           plotobj.linewidth=str2double(nums{ii});
       end
    end   
end  
 
 

 
 
